@@ -1,9 +1,14 @@
 # frozen_string_literal: true
 
+require "securerandom"
+
+require "active_support/core_ext/module/delegation"
+require "active_support/core_ext/string/starts_ends_with"
+require "active_support/core_ext/string/inflections"
+
 module Labkit
   # A context can be used to provide structured information on what resources
   # GitLab is working on within a service.
-  # The currently supported keys are defined in the `KNOWN_KEYS` constant.
   #
   # Values can be provided by passing a hash. If one of the values is a Proc
   # the proc will only be called when the value is actually needed.
@@ -21,8 +26,7 @@ module Labkit
     LOG_KEY = "meta"
     CORRELATION_ID_KEY = "correlation_id"
     RAW_KEYS = [CORRELATION_ID_KEY].freeze
-    KNOWN_KEYS = %w[user project root_namespace subscription_plan caller_id
-                    related_class feature_category].freeze
+    HEADER_PREFIX = "X-Gitlab-"
 
     class << self
       def with_context(attributes = {})
@@ -67,6 +71,10 @@ module Labkit
         @known_log_keys ||= (KNOWN_KEYS.map(&method(:log_key)) + RAW_KEYS).freeze
       end
 
+      def header_name(name)
+        HEADER_PREFIX + log_key(name).titlecase(keep_id_suffix: true).gsub(/\W/, "-")
+      end
+
       private
 
       def contexts
@@ -95,16 +103,27 @@ module Labkit
       data[CORRELATION_ID_KEY]
     end
 
+    def to_headers
+      to_h.except(CORRELATION_ID_KEY).transform_keys do |key|
+        self.class.header_name(key)
+      end
+    end
+
+    def get_attribute(attribute)
+      raw = call_or_value(data[log_key(attribute)])
+
+      call_or_value(raw)
+    end
+
     protected
 
     def assign_attributes(attributes)
       attributes = attributes.transform_keys(&method(:log_key))
-      attributes = attributes.slice(*known_log_keys)
 
       data.merge!(attributes)
 
       # Remove keys that had their values set to `nil` in the new attributes
-      data.keep_if { |_, value| value.present? }
+      data.keep_if { |_, value| valid_data?(value) }
 
       # Assign a correlation if it was missing in the first context or when
       # explicitly removed
@@ -115,20 +134,28 @@ module Labkit
 
     private
 
-    delegate :log_key, :known_log_keys, to: :class
+    delegate :log_key, to: :class
 
     attr_reader :data
 
+    def call_or_value(value)
+      value.respond_to?(:call) ? value.call : value
+    end
+
     def expand_data
       data.transform_values do |value|
-        value = value.respond_to?(:call) ? value.call : value
+        value = call_or_value(value)
 
-        value.presence
+        value if valid_data?(value)
       end.compact
     end
 
     def new_id
       SecureRandom.hex
     end
+
+    def valid_data?(value)
+      value == false || value.present?
+    end
   end
 end
