@@ -7,7 +7,7 @@ module Labkit
         # ActiveRecord bridges active record notifications to
         # the distributed tracing subsystem
         class Subscriber
-          include RailsCommon
+          include Labkit::Tracing::TracingCommon
 
           ACTIVE_RECORD_NOTIFICATION_TOPIC = "sql.active_record"
 
