@@ -7,7 +7,7 @@ module Labkit
         # ActiveSupport bridges action active support notifications to
         # the distributed tracing subsystem
         class Subscriber
-          include RailsCommon
+          include Labkit::Tracing::TracingCommon
 
           CACHE_READ_TOPIC = "cache_read.active_support"
           CACHE_GENERATE_TOPIC = "cache_generate.active_support"
